@@ -16,7 +16,7 @@ from dotdrop.settings import Settings
 from dotdrop.profile import Profile
 from dotdrop.action import Action, Transform
 from dotdrop.logger import Logger
-from dotdrop.utils import strip_home
+from dotdrop.utils import strip_home, debug_list, debug_dict
 from dotdrop.exceptions import UndefinedException
 
 
@@ -24,6 +24,7 @@ TILD = '~'
 
 
 class CfgAggregator:
+    """The config aggregator class"""
 
     file_prefix = 'f'
     dir_prefix = 'd'
@@ -40,7 +41,7 @@ class CfgAggregator:
         self.profile_key = profile_key
         self.debug = debug
         self.dry = dry
-        self.log = Logger()
+        self.log = Logger(debug=self.debug)
         self._load()
 
     ########################################################
@@ -73,7 +74,7 @@ class CfgAggregator:
 
         key = dotfile.key
         ret = self.cfgyaml.add_dotfile_to_profile(key, self.profile_key)
-        if ret and self.debug:
+        if ret:
             msg = 'new dotfile {} to profile {}'
             self.log.dbg(msg.format(key, self.profile_key))
 
@@ -106,10 +107,10 @@ class CfgAggregator:
         """
         dotfiles = []
         dst = self._norm_path(dst)
-        for d in self.dotfiles:
-            left = self._norm_path(d.dst)
+        for dotfile in self.dotfiles:
+            left = self._norm_path(dotfile.dst)
             if left == dst:
-                dotfiles.append(d)
+                dotfiles.append(dotfile)
         return dotfiles
 
     def get_dotfile_by_src_dst(self, src, dst):
@@ -120,14 +121,14 @@ class CfgAggregator:
         """
         try:
             src = self.cfgyaml.resolve_dotfile_src(src)
-        except UndefinedException as e:
+        except UndefinedException as exc:
             err = 'unable to resolve {}: {}'
-            self.log.err(err.format(src, e))
+            self.log.err(err.format(src, exc))
             return None
         dotfiles = self.get_dotfile_by_dst(dst)
-        for d in dotfiles:
-            if d.src == src:
-                return d
+        for dotfile in dotfiles:
+            if dotfile.src == src:
+                return dotfile
         return None
 
     def save(self):
@@ -162,10 +163,10 @@ class CfgAggregator:
     def get_profiles_by_dotfile_key(self, key):
         """return all profiles having this dotfile"""
         res = []
-        for p in self.profiles:
-            keys = [d.key for d in p.dotfiles]
+        for profile in self.profiles:
+            keys = [dotfile.key for dotfile in profile.dotfiles]
             if key in keys:
-                res.append(p)
+                res.append(profile)
         return res
 
     def get_dotfiles(self):
@@ -195,8 +196,7 @@ class CfgAggregator:
         """create a new dotfile"""
         # get a new dotfile with a unique key
         key = self._get_new_dotfile_key(dst)
-        if self.debug:
-            self.log.dbg('new dotfile key: {}'.format(key))
+        self.log.dbg('new dotfile key: {}'.format(key))
         # add the dotfile
         if not self.cfgyaml.add_dotfile(key, src, dst, link, chmod=chmod):
             return None
@@ -217,33 +217,27 @@ class CfgAggregator:
 
         # dotfiles
         self.dotfiles = Dotfile.parse_dict(self.cfgyaml.dotfiles)
-        if self.debug:
-            self._debug_list('dotfiles', self.dotfiles)
+        debug_list('dotfiles', self.dotfiles, self.debug)
 
         # profiles
         self.profiles = Profile.parse_dict(self.cfgyaml.profiles)
-        if self.debug:
-            self._debug_list('profiles', self.profiles)
+        debug_list('profiles', self.profiles, self.debug)
 
         # actions
         self.actions = Action.parse_dict(self.cfgyaml.actions)
-        if self.debug:
-            self._debug_list('actions', self.actions)
+        debug_list('actions', self.actions, self.debug)
 
         # trans_r
         self.trans_r = Transform.parse_dict(self.cfgyaml.trans_r)
-        if self.debug:
-            self._debug_list('trans_r', self.trans_r)
+        debug_list('trans_r', self.trans_r, self.debug)
 
         # trans_w
         self.trans_w = Transform.parse_dict(self.cfgyaml.trans_w)
-        if self.debug:
-            self._debug_list('trans_w', self.trans_w)
+        debug_list('trans_w', self.trans_w, self.debug)
 
         # variables
         self.variables = self.cfgyaml.variables
-        if self.debug:
-            self._debug_dict('variables', self.variables)
+        debug_dict('variables', self.variables, self.debug)
 
         # patch dotfiles in profiles
         self._patch_keys_to_objs(self.profiles,
@@ -259,9 +253,9 @@ class CfgAggregator:
         # patch actions in settings default_actions
         self._patch_keys_to_objs([self.settings],
                                  "default_actions", self._get_action_w_args)
-        if self.debug:
-            msg = 'default actions: {}'.format(self.settings.default_actions)
-            self.log.dbg(msg)
+
+        msg = 'default actions: {}'.format(self.settings.default_actions)
+        self.log.dbg(msg)
 
         # patch trans_w/trans_r in dotfiles
         self._patch_keys_to_objs(self.dotfiles,
@@ -280,29 +274,25 @@ class CfgAggregator:
         """
         if not containers:
             return
-        if self.debug:
-            self.log.dbg('patching {} ...'.format(keys))
-        for c in containers:
+        self.log.dbg('patching {} ...'.format(keys))
+        for container in containers:
             objects = []
-            okeys = getattr(c, keys)
+            okeys = getattr(container, keys)
             if not okeys:
                 continue
             if not islist:
                 okeys = [okeys]
-            for k in okeys:
-                o = get_by_key(k)
-                if not o:
-                    err = '{} does not contain'.format(c)
-                    err += ' a {} entry named {}'.format(keys, k)
+            for key in okeys:
+                obj = get_by_key(key)
+                if not obj:
+                    err = '{} does not contain'.format(container)
+                    err += ' a {} entry named {}'.format(keys, key)
                     self.log.err(err)
                     raise Exception(err)
-                objects.append(o)
+                objects.append(obj)
             if not islist:
                 objects = objects[0]
-            # if self.debug:
-            #     er = 'patching {}.{} with {}'
-            #     self.log.dbg(er.format(c, keys, objects))
-            setattr(c, keys, objects)
+            setattr(container, keys, objects)
 
     ########################################################
     # dotfile key
@@ -316,7 +306,8 @@ class CfgAggregator:
             return self._get_long_key(path, existing_keys)
         return self._get_short_key(path, existing_keys)
 
-    def _norm_key_elem(self, elem):
+    @classmethod
+    def _norm_key_elem(cls, elem):
         """normalize path element for sanity"""
         elem = elem.lstrip('.')
         elem = elem.replace(' ', '-')
@@ -341,8 +332,8 @@ class CfgAggregator:
         dirs.reverse()
         prefix = self.dir_prefix if os.path.isdir(path) else self.file_prefix
         entries = []
-        for d in dirs:
-            entries.insert(0, d)
+        for dri in dirs:
+            entries.insert(0, dri)
             key = self.key_sep.join([prefix] + entries)
             if key not in keys:
                 return key
@@ -367,14 +358,14 @@ class CfgAggregator:
         if self.dry:
             return
         self.save()
-        if self.debug:
-            self.log.dbg('reloading config')
+        self.log.dbg('reloading config')
         olddebug = self.debug
         self.debug = False
         self._load()
         self.debug = olddebug
 
-    def _norm_path(self, path):
+    @classmethod
+    def _norm_path(cls, path):
         if not path:
             return path
         path = os.path.expanduser(path)
@@ -384,12 +375,12 @@ class CfgAggregator:
 
     def _split_path_for_key(self, path):
         """return a list of path elements, excluded home path"""
-        p = strip_home(path)
+        path = strip_home(path)
         dirs = []
         while True:
-            p, f = os.path.split(p)
-            dirs.append(f)
-            if not p or not f:
+            path, file = os.path.split(path)
+            dirs.append(file)
+            if not path or not file:
                 break
         dirs.reverse()
         # remove empty entries
@@ -410,9 +401,8 @@ class CfgAggregator:
         if len(fields) > 1:
             # we have args
             key, *args = fields
-            if self.debug:
-                msg = 'action with parm: {} and {}'
-                self.log.dbg(msg.format(key, args))
+            msg = 'action with parm: {} and {}'
+            self.log.dbg(msg.format(key, args))
             action = self._get_action(key).copy(args)
         else:
             action = self._get_action(key)
@@ -425,9 +415,8 @@ class CfgAggregator:
             if len(fields) > 1:
                 # we have args
                 key, *args = fields
-                if self.debug:
-                    msg = 'trans with parm: {} and {}'
-                    self.log.dbg(msg.format(key, args))
+                msg = 'trans with parm: {} and {}'
+                self.log.dbg(msg.format(key, args))
                 trans = getter(key).copy(args)
             else:
                 trans = getter(key)
@@ -447,19 +436,3 @@ class CfgAggregator:
             return next(x for x in self.trans_w if x.key == key)
         except StopIteration:
             return None
-
-    def _debug_list(self, title, elems):
-        """pretty print list"""
-        if not self.debug:
-            return
-        self.log.dbg('{}:'.format(title))
-        for e in elems:
-            self.log.dbg('\t- {}'.format(e))
-
-    def _debug_dict(self, title, elems):
-        """pretty print dict"""
-        if not self.debug:
-            return
-        self.log.dbg('{}:'.format(title))
-        for k, v in elems.items():
-            self.log.dbg('\t- \"{}\": {}'.format(k, v))
