@@ -22,10 +22,11 @@ TILD = '~'
 
 
 class Updater:
+    """dotfiles updater"""
 
     def __init__(self, dotpath, variables, conf,
                  dry=False, safe=True, debug=False,
-                 ignore=[], showpatch=False,
+                 ignore=None, showpatch=False,
                  ignore_missing_in_dotdrop=False):
         """constructor
         @dotpath: path where dotfiles are stored
@@ -43,7 +44,7 @@ class Updater:
         self.dry = dry
         self.safe = safe
         self.debug = debug
-        self.ignore = ignore
+        self.ignore = ignore or []
         self.ignores = None
         self.showpatch = showpatch
         self.ignore_missing_in_dotdrop = ignore_missing_in_dotdrop
@@ -52,7 +53,7 @@ class Updater:
                                      debug=self.debug)
         # save template vars
         self.tvars = self.templater.add_tmp_vars()
-        self.log = Logger()
+        self.log = Logger(debug=self.debug)
 
     def update_path(self, path):
         """update the dotfile installed on path"""
@@ -69,9 +70,8 @@ class Updater:
                 self.log.err(msg.format(dotfile.key))
                 return False
 
-            if self.debug:
-                msg = 'updating {} from path \"{}\"'
-                self.log.dbg(msg.format(dotfile, path))
+            msg = 'updating {} from path \"{}\"'
+            self.log.dbg(msg.format(dotfile, path))
             if not self._update(path, dotfile):
                 return False
         return True
@@ -81,8 +81,7 @@ class Updater:
         dotfile = self.conf.get_dotfile(key)
         if not dotfile:
             return False
-        if self.debug:
-            self.log.dbg('updating {} from key \"{}\"'.format(dotfile, key))
+        self.log.dbg('updating {} from key \"{}\"'.format(dotfile, key))
         path = self.conf.path_to_dotfile_dst(dotfile.dst)
         return self._update(path, dotfile)
 
@@ -92,8 +91,7 @@ class Updater:
         new_path = None
         ignores = list(set(self.ignore + dotfile.upignore))
         self.ignores = patch_ignores(ignores, dotfile.dst, debug=self.debug)
-        if self.debug:
-            self.log.dbg('ignore pattern(s): {}'.format(self.ignores))
+        self.log.dbg('ignore pattern(s): {}'.format(self.ignores))
 
         deployed_path = os.path.expanduser(path)
         local_path = os.path.join(self.dotpath, dotfile.src)
@@ -128,15 +126,13 @@ class Updater:
         if os.path.isdir(new_path):
             ret = self._handle_dir(new_path, local_path, dotfile)
         else:
-            ret = self._handle_file(new_path, local_path, dotfile)
+            ret = self._handle_file(new_path, local_path)
 
         if deployed_mode != local_mode:
             # mirror rights
-            if self.debug:
-                m = 'adopt mode {:o} for {}'
-                self.log.dbg(m.format(deployed_mode, dotfile.key))
-            r = self.conf.update_dotfile(dotfile.key, deployed_mode)
-            if r:
+            msg = 'adopt mode {:o} for {}'
+            self.log.dbg(msg.format(deployed_mode, dotfile.key))
+            if self.conf.update_dotfile(dotfile.key, deployed_mode):
                 ret = True
 
         # clean temporary files
@@ -149,8 +145,7 @@ class Updater:
         trans = dotfile.get_trans_w()
         if not trans:
             return path
-        if self.debug:
-            self.log.dbg('executing write transformation {}'.format(trans))
+        self.log.dbg('executing write transformation {}'.format(trans))
         tmp = get_unique_tmp_name()
         self.templater.restore_vars(self.tvars)
         newvars = dotfile.get_dotfile_variables()
@@ -166,8 +161,7 @@ class Updater:
 
     def _is_template(self, path):
         if not Templategen.is_template(path, ignore=self.ignores):
-            if self.debug:
-                self.log.dbg('{} is NO template'.format(path))
+            self.log.dbg('{} is NO template'.format(path))
             return False
         self.log.warn('{} uses template, update manually'.format(path))
         return True
@@ -192,8 +186,8 @@ class Updater:
             lefts = get_file_perm(left)
             rights = get_file_perm(right)
             return lefts == rights
-        except OSError as e:
-            self.log.err(e)
+        except OSError as exc:
+            self.log.err(exc)
             return False
 
     def _mirror_rights(self, src, dst):
@@ -201,35 +195,32 @@ class Updater:
         dstr = get_file_perm(dst)
         if srcr == dstr:
             return
-        if self.debug:
-            msg = 'copy rights from {} ({:o}) to {} ({:o})'
-            self.log.dbg(msg.format(src, srcr, dst, dstr))
+        msg = 'copy rights from {} ({:o}) to {} ({:o})'
+        self.log.dbg(msg.format(src, srcr, dst, dstr))
         try:
             mirror_file_rights(src, dst)
-        except OSError as e:
-            self.log.err(e)
+        except OSError as exc:
+            self.log.err(exc)
 
-    def _handle_file(self, deployed_path, local_path, dotfile, compare=True):
+    def _handle_file(self, deployed_path, local_path, compare=True):
         """sync path (deployed file) and local_path (dotdrop dotfile path)"""
         if self._ignore([deployed_path, local_path]):
             self.log.sub('\"{}\" ignored'.format(local_path))
             return True
-        if self.debug:
-            self.log.dbg('update for file {} and {}'.format(
-                deployed_path,
-                local_path,
-            ))
+        self.log.dbg('update for file {} and {}'.format(
+            deployed_path,
+            local_path,
+        ))
         if self._is_template(local_path):
             # dotfile is a template
-            if self.debug:
-                self.log.dbg('{} is a template'.format(local_path))
+            self.log.dbg('{} is a template'.format(local_path))
             if self.showpatch:
                 try:
                     self._show_patch(deployed_path, local_path)
-                except UndefinedException as e:
+                except UndefinedException as exc:
                     msg = 'unable to show patch for {}: {}'.format(
                         deployed_path,
-                        e,
+                        exc,
                     )
                     self.log.warn(msg)
             return False
@@ -237,11 +228,10 @@ class Updater:
                 filecmp.cmp(deployed_path, local_path, shallow=False) and \
                 self._same_rights(deployed_path, local_path):
             # no difference
-            if self.debug:
-                self.log.dbg('identical files: {} and {}'.format(
-                    deployed_path,
-                    local_path,
-                ))
+            self.log.dbg('identical files: {} and {}'.format(
+                deployed_path,
+                local_path,
+            ))
             return True
         if not self._overwrite(deployed_path, local_path):
             return False
@@ -252,26 +242,24 @@ class Updater:
                     local_path,
                 ))
             else:
-                if self.debug:
-                    self.log.dbg('cp {} {}'.format(deployed_path, local_path))
+                self.log.dbg('cp {} {}'.format(deployed_path, local_path))
                 shutil.copyfile(deployed_path, local_path)
                 self._mirror_rights(deployed_path, local_path)
                 self.log.sub('\"{}\" updated'.format(local_path))
-        except IOError as e:
+        except IOError as exc:
             self.log.warn('{} update failed, do manually: {}'.format(
                 deployed_path,
-                e
+                exc
             ))
             return False
         return True
 
     def _handle_dir(self, deployed_path, local_path, dotfile):
         """sync path (local dir) and local_path (dotdrop dir path)"""
-        if self.debug:
-            self.log.dbg('handle update for dir {} to {}'.format(
-                deployed_path,
-                local_path,
-            ))
+        self.log.dbg('handle update for dir {} to {}'.format(
+            deployed_path,
+            local_path,
+        ))
         # paths must be absolute (no tildes)
         deployed_path = os.path.expanduser(deployed_path)
         local_path = os.path.expanduser(local_path)
@@ -286,18 +274,9 @@ class Updater:
         self._mirror_rights(deployed_path, local_path)
         return ret
 
-    def _merge_dirs(self, diff, dotfile):
-        """Synchronize directories recursively."""
-        left, right = diff.left, diff.right
-        if self.debug:
-            self.log.dbg('sync dir {} to {}'.format(left, right))
-        if self._ignore([left, right]):
-            return True
-
-        ignore_missing_in_dotdrop = self.ignore_missing_in_dotdrop or \
-            dotfile.ignore_missing_in_dotdrop
-
-        # create dirs that don't exist in dotdrop
+    def _merge_dirs_create_left_only(self, diff, left, right,
+                                     ignore_missing_in_dotdrop):
+        """create dirs that don't exist in dotdrop"""
         for toadd in diff.left_only:
             exist = os.path.join(left, toadd)
             if not os.path.isdir(exist):
@@ -312,11 +291,10 @@ class Updater:
             if self.dry:
                 self.log.dry('would cp -r {} {}'.format(exist, new))
                 continue
-            if self.debug:
-                self.log.dbg('cp -r {} {}'.format(exist, new))
+            self.log.dbg('cp -r {} {}'.format(exist, new))
 
             # Newly created directory should be copied as is (for efficiency).
-            def ig(src, names):
+            def ign(src, names):
                 whitelist, blacklist = set(), set()
                 for ignore in self.ignores:
                     for name in names:
@@ -330,10 +308,17 @@ class Updater:
                             blacklist.add(name)
                 return blacklist - whitelist
 
-            shutil.copytree(exist, new, ignore=ig)
+            try:
+                shutil.copytree(exist, new, ignore=ign)
+            except OSError as exc:
+                msg = 'error copying dir {}'.format(exist)
+                self.log.err('{}: {}'.format(msg, exc))
+                continue
             self.log.sub('\"{}\" dir added'.format(new))
 
-        # remove dirs that don't exist in deployed version
+    def _merge_dirs_remove_right_only(self, diff, left, right,
+                                      ignore_missing_in_dotdrop):
+        """remove dirs that don't exist in deployed version"""
         for toremove in diff.right_only:
             old = os.path.join(right, toremove)
             if not os.path.isdir(old):
@@ -344,8 +329,7 @@ class Updater:
             if self.dry:
                 self.log.dry('would rm -r {}'.format(old))
                 continue
-            if self.debug:
-                self.log.dbg('rm -r {}'.format(old))
+            self.log.dbg('rm -r {}'.format(old))
             if not self._confirm_rm_r(old):
                 continue
             removepath(old, logger=self.log)
@@ -356,20 +340,21 @@ class Updater:
         fdiff = diff.diff_files
         fdiff.extend(diff.funny_files)
         fdiff.extend(diff.common_funny)
-        for f in fdiff:
-            fleft = os.path.join(left, f)
-            fright = os.path.join(right, f)
+        for file in fdiff:
+            fleft = os.path.join(left, file)
+            fright = os.path.join(right, file)
             if (ignore_missing_in_dotdrop and not os.path.exists(fright)) or \
                     self._ignore([fleft, fright]):
                 continue
             if self.dry:
                 self.log.dry('would cp {} {}'.format(fleft, fright))
                 continue
-            if self.debug:
-                self.log.dbg('cp {} {}'.format(fleft, fright))
-            self._handle_file(fleft, fright, dotfile, compare=False)
+            self.log.dbg('cp {} {}'.format(fleft, fright))
+            self._handle_file(fleft, fright, compare=False)
 
-        # copy files that don't exist in dotdrop
+    def _merge_dirs_copy_left_only(self, diff, left, right,
+                                   ignore_missing_in_dotdrop):
+        """copy files that don't exist in dotdrop"""
         for toadd in diff.left_only:
             exist = os.path.join(left, toadd)
             if os.path.isdir(exist):
@@ -382,13 +367,19 @@ class Updater:
             if self.dry:
                 self.log.dry('would cp {} {}'.format(exist, new))
                 continue
-            if self.debug:
-                self.log.dbg('cp {} {}'.format(exist, new))
-            shutil.copyfile(exist, new)
+            self.log.dbg('cp {} {}'.format(exist, new))
+            try:
+                shutil.copyfile(exist, new)
+            except OSError as exc:
+                msg = 'error copying file {}'.format(exist)
+                self.log.err('{}: {}'.format(msg, exc))
+                continue
+
             self._mirror_rights(exist, new)
             self.log.sub('\"{}\" added'.format(new))
 
-        # remove files that don't exist in deployed version
+    def _merge_dirs_remove_right_only_2(self, diff, right):
+        """remove files that don't exist in deployed version"""
         for toremove in diff.right_only:
             new = os.path.join(right, toremove)
             if not os.path.exists(new):
@@ -401,11 +392,28 @@ class Updater:
             if self.dry:
                 self.log.dry('would rm {}'.format(new))
                 continue
-            if self.debug:
-                self.log.dbg('rm {}'.format(new))
+            self.log.dbg('rm {}'.format(new))
             removepath(new, logger=self.log)
             self.log.sub('\"{}\" removed'.format(new))
 
+    def _merge_dirs(self, diff, dotfile):
+        """Synchronize directories recursively."""
+        left, right = diff.left, diff.right
+        self.log.dbg('sync dir {} to {}'.format(left, right))
+        if self._ignore([left, right]):
+            return True
+
+        ignore_missing_in_dotdrop = self.ignore_missing_in_dotdrop or \
+            dotfile.ignore_missing_in_dotdrop
+
+        self._merge_dirs_create_left_only(diff, left, right,
+                                          ignore_missing_in_dotdrop)
+        self._merge_dirs_remove_right_only(diff, left, right,
+                                           ignore_missing_in_dotdrop)
+        self._merge_dirs_copy_left_only(diff, left, right,
+                                        ignore_missing_in_dotdrop)
+        self._merge_dirs_remove_right_only_2(diff, right)
+
         # compare rights
         for common in diff.common_files:
             leftf = os.path.join(left, common)
@@ -436,7 +444,6 @@ class Updater:
 
     def _ignore(self, paths):
         if must_ignore(paths, self.ignores, debug=self.debug):
-            if self.debug:
-                self.log.dbg('ignoring update for {}'.format(paths))
+            self.log.dbg('ignoring update for {}'.format(paths))
             return True
         return False
