@@ -18,6 +18,7 @@ from shutil import rmtree, which
 
 # local import
 from dotdrop.logger import Logger
+from dotdrop.exceptions import UnmetDependency
 
 LOG = Logger()
 STAR = '*'
@@ -37,11 +38,11 @@ NOREMOVE = [os.path.normpath(p) for p in DONOTDELETE]
 def run(cmd, debug=False):
     """run a command (expects a list)"""
     if debug:
-        LOG.dbg('exec: {}'.format(' '.join(cmd)))
-    p = subprocess.Popen(cmd, shell=False,
-                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-    out, _ = p.communicate()
-    ret = p.returncode
+        LOG.dbg('exec: {}'.format(' '.join(cmd)), force=True)
+    proc = subprocess.Popen(cmd, shell=False,
+                            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    out, _ = proc.communicate()
+    ret = proc.returncode
     out = out.splitlines(keepends=True)
     lines = ''.join([x.decode('utf-8', 'replace') for x in out])
     return ret == 0, lines
@@ -50,21 +51,21 @@ def run(cmd, debug=False):
 def write_to_tmpfile(content):
     """write some content to a tmp file"""
     path = get_tmpfile()
-    with open(path, 'wb') as f:
-        f.write(content)
+    with open(path, 'wb') as file:
+        file.write(content)
     return path
 
 
-def shell(cmd, debug=False):
+def shellrun(cmd, debug=False):
     """
     run a command in the shell (expects a string)
     returns True|False, output
     """
     if debug:
-        LOG.dbg('shell exec: \"{}\"'.format(cmd))
+        LOG.dbg('shell exec: \"{}\"'.format(cmd), force=True)
     ret, out = subprocess.getstatusoutput(cmd)
     if debug:
-        LOG.dbg('shell result ({}): {}'.format(ret, out))
+        LOG.dbg('shell result ({}): {}'.format(ret, out), force=True)
     return ret == 0, out
 
 
@@ -92,25 +93,27 @@ def diff(original, modified,
 
 def get_tmpdir():
     """create and return the temporary directory"""
+# pylint: disable=W0603
     global TMPDIR
+# pylint: enable=W0603
     if TMPDIR:
         return TMPDIR
-    t = _get_tmpdir()
-    TMPDIR = t
-    return t
+    tmp = _get_tmpdir()
+    TMPDIR = tmp
+    return tmp
 
 
 def _get_tmpdir():
     """create the tmpdir"""
     try:
         if ENV_TEMP in os.environ:
-            t = os.environ[ENV_TEMP]
-            t = os.path.expanduser(t)
-            t = os.path.abspath(t)
-            t = os.path.normpath(t)
-            os.makedirs(t, exist_ok=True)
-            return t
-    except Exception:
+            tmp = os.environ[ENV_TEMP]
+            tmp = os.path.expanduser(tmp)
+            tmp = os.path.abspath(tmp)
+            tmp = os.path.normpath(tmp)
+            os.makedirs(tmp, exist_ok=True)
+            return tmp
+    except OSError:
         pass
     return tempfile.mkdtemp(prefix='dotdrop-')
 
@@ -159,12 +162,12 @@ def removepath(path, logger=None):
         else:
             err = 'Unsupported file type for deletion: {}'.format(path)
             raise OSError(err)
-    except Exception as e:
-        err = str(e)
+    except Exception as exc:
+        err = str(exc)
         if logger:
             logger.warn(err)
             return
-        raise OSError(err)
+        raise OSError(err) from exc
 
 
 def samefile(path1, path2):
@@ -203,53 +206,57 @@ def must_ignore(paths, ignores, debug=False):
     if not ignores:
         return False
     if debug:
-        LOG.dbg('must ignore? \"{}\" against {}'.format(paths, ignores))
+        LOG.dbg('must ignore? \"{}\" against {}'.format(paths, ignores),
+                force=True)
     ignored_negative, ignored = categorize(
         lambda ign: ign.startswith('!'), ignores)
-    for p in paths:
+    for path in paths:
         ignore_matches = []
         # First ignore dotfiles
         for i in ignored:
-            if fnmatch.fnmatch(p, i):
+            if fnmatch.fnmatch(path, i):
                 if debug:
-                    LOG.dbg('ignore \"{}\" match: {}'.format(i, p))
-                ignore_matches.append(p)
+                    LOG.dbg('ignore \"{}\" match: {}'.format(i, path),
+                            force=True)
+                ignore_matches.append(path)
         # Then remove any matches that actually shouldn't be ignored
-        for ni in ignored_negative:
+        for nign in ignored_negative:
             # Each of these will start with an '!' so we need to remove that
-            ni = ni[1:]
-            if fnmatch.fnmatch(p, ni):
+            nign = nign[1:]
+            if fnmatch.fnmatch(path, nign):
                 if debug:
-                    LOG.dbg('negative ignore \"{}\" match: {}'.format(ni, p))
+                    msg = 'negative ignore \"{}\" match: {}'.format(nign, path)
+                    LOG.dbg(msg, force=True)
                 try:
-                    ignore_matches.remove(p)
+                    ignore_matches.remove(path)
                 except ValueError:
-                    LOG.warn('no files that are currently being ignored match '
-                             '\"{}\". In order for a negative ignore pattern '
-                             'to work, it must match a file that is being '
-                             'ignored by a previous ignore pattern.'.format(ni)
+                    LOG.warn('no files that are currently being '
+                             'ignored match \"{}\". In order '
+                             'for a negative ignore pattern '
+                             'to work, it must match a file '
+                             'that is being ignored by a '
+                             'previous ignore pattern.'.format(nign)
                              )
         if ignore_matches:
             return True
     if debug:
-        LOG.dbg('NOT ignoring {}'.format(paths))
+        LOG.dbg('NOT ignoring {}'.format(paths), force=True)
     return False
 
 
 def uniq_list(a_list):
     """unique elements of a list while preserving order"""
     new = []
-    for a in a_list:
-        if a not in new:
-            new.append(a)
+    for elem in a_list:
+        if elem not in new:
+            new.append(elem)
     return new
 
 
 def patch_ignores(ignores, prefix, debug=False):
     """allow relative ignore pattern"""
     new = []
-    if debug:
-        LOG.dbg('ignores before patching: {}'.format(ignores))
+    LOG.dbg('ignores before patching: {}'.format(ignores), force=debug)
     for ignore in ignores:
         negative = ignore.startswith('!')
         if negative:
@@ -276,16 +283,15 @@ def patch_ignores(ignores, prefix, debug=False):
             new.append('!' + path)
         else:
             new.append(path)
-    if debug:
-        LOG.dbg('ignores after patching: {}'.format(new))
+    LOG.dbg('ignores after patching: {}'.format(new), force=debug)
     return new
 
 
 def get_module_functions(mod):
     """return a list of fonction from a module"""
     funcs = []
-    for m in inspect.getmembers(mod):
-        name, func = m
+    for memb in inspect.getmembers(mod):
+        name, func = memb
         if not inspect.isfunction(func):
             continue
         funcs.append((name, func))
@@ -297,8 +303,12 @@ def get_module_from_path(path):
     if not path or not os.path.exists(path):
         return None
     module_name = os.path.basename(path).rstrip('.py')
-    loader = importlib.machinery.SourceFileLoader(module_name, path)
-    mod = loader.load_module()
+    # allow any type of files
+    importlib.machinery.SOURCE_SUFFIXES.append('')
+    # import module
+    spec = importlib.util.spec_from_file_location(module_name, path)
+    mod = importlib.util.module_from_spec(spec)
+    spec.loader.exec_module(mod)
     return mod
 
 
@@ -309,14 +319,15 @@ def dependencies_met():
     err = 'The tool \"{}\" was not found in the PATH!'
     for dep in deps:
         if not which(dep):
-            raise Exception(err.format(dep))
+            raise UnmetDependency(err.format(dep))
     # check python deps
     err = 'missing python module \"{}\"'
 
+# pylint: disable=C0415
     # python-magic
     try:
         import magic
-        assert(magic)
+        assert magic
         if not hasattr(magic, 'from_file'):
             LOG.warn(err.format('python-magic'))
     except ImportError:
@@ -325,23 +336,24 @@ def dependencies_met():
     # docopt
     try:
         from docopt import docopt
-        assert(docopt)
-    except ImportError:
-        raise Exception(err.format('docopt'))
+        assert docopt
+    except ImportError as exc:
+        raise Exception(err.format('docopt')) from exc
 
     # jinja2
     try:
         import jinja2
-        assert(jinja2)
-    except ImportError:
-        raise Exception(err.format('jinja2'))
+        assert jinja2
+    except ImportError as exc:
+        raise Exception(err.format('jinja2')) from exc
 
     # ruamel.yaml
     try:
         from ruamel.yaml import YAML
-        assert(YAML)
-    except ImportError:
-        raise Exception(err.format('ruamel.yaml'))
+        assert YAML
+    except ImportError as exc:
+        raise Exception(err.format('ruamel.yaml')) from exc
+# pylint: enable=C0415
 
 
 def mirror_file_rights(src, dst):
@@ -374,13 +386,15 @@ def get_file_perm(path):
 
 
 def chmod(path, mode, debug=False):
+    """change mode of file"""
     if debug:
-        LOG.dbg('chmod {} {}'.format(oct(mode), path))
+        LOG.dbg('chmod {} {}'.format(oct(mode), path), force=True)
     os.chmod(path, mode)
     return get_file_perm(path) == mode
 
 
 def adapt_workers(options, logger):
+    """adapt number of workers if safe/dry"""
     if options.safe and options.workers > 1:
         logger.warn('workers set to 1 when --force is not used')
         options.workers = 1
@@ -396,3 +410,26 @@ def categorize(function, iterable):
     element"""
     return (tuple(filter(function, iterable)),
             tuple(itertools.filterfalse(function, iterable)))
+
+
+def debug_list(title, elems, debug):
+    """pretty print list"""
+    if not debug:
+        return
+    LOG.dbg('{}:'.format(title))
+    for elem in elems:
+        LOG.dbg('\t- {}'.format(elem))
+
+
+def debug_dict(title, elems, debug):
+    """pretty print dict"""
+    if not debug:
+        return
+    LOG.dbg('{}:'.format(title))
+    for k, val in elems.items():
+        if isinstance(val, list):
+            LOG.dbg('\t- \"{}\":'.format(k))
+            for i in val:
+                LOG.dbg('\t\t- {}'.format(i))
+        else:
+            LOG.dbg('\t- \"{}\": {}'.format(k, val))
